{
  description = "A Nix flake for Gargantext";
  # To use this Nix flake you may need to enable Nix flake support for your user with:
  #     echo >>~/.config/nix/nix.conf "experimental-features = nix-command flakes"

  inputs = {
    haskellNix.url = "github:input-output-hk/haskell.nix";

    # For trying to hit cache.iog.io one has to follow haskell.nix's nixpkgs-unstable pins.
    # However picking an out-of-date compiler-nix-name may render this useless.
    nixpkgs.follows = "haskellNix/nixpkgs-unstable";

    # TODO: not really necessary, flake-utils.lib.eachSystem may be inlined in this flake.nix.
    flake-utils.url = "github:numtide/flake-utils";

    # FIXME: git.iscpif.fr's gitlab's API does not work,
    # hence fallback to the git+https:// scheme.
    #gargantext-prelude.url = "gitlab:gargantext/haskell-gargantext-prelude?host=git.iscpif.fr";
    gargantext-prelude = { url = "git+https://gitlab.iscpif.fr/gargantext/haskell-gargantext-prelude.git"; flake = false; };
    # WARNING: using the debugNaN branch
    gargantext-graph = { url = "git+https://gitlab.iscpif.fr/gargantext/gargantext-graph.git?ref=debugNaN"; flake = false; };
    hlcm = { url = "git+https://gitlab.iscpif.fr/gargantext/hlcm.git"; flake = false; };
    rdf4h = { url = "github:robstewart57/rdf4h"; flake = false; };

    # WARNING: using the wavewave/ghcHEAD branch
    ekg-json = { url = "github:MercuryTechnologies/ekg-json/wavewave/ghcHEAD"; flake = false; };

    iso639 = { url = "git+https://gitlab.iscpif.fr/gargantext/iso639.git"; flake = false; };
    crawlerArxiv = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/arxiv-api.git"; flake = false; };
    crawlerHAL = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/hal.git"; flake = false; };
    crawlerIsidore = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/isidore.git"; flake = false; };
    #crawlerIsidore = { url = "git+file:///home/julm/src/haskell/gargantext/git/haskell.nix/isidore"; flake = false; };
    crawlerISTEX = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/istex.git"; flake = false; };
    crawlerPubMed = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/pubmed.git"; flake = false; };
    openalex = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/openalex.git"; flake = false; };
    epo-api-client = { url = "git+https://gitlab.iscpif.fr/gargantext/crawlers/epo-proxy-api.git"; flake = false; };

    # Data Mining Libs
    data-time-segment = { url = "github:delanoe/data-time-segment"; flake = false; };

    # Graph libs
    haskell-igraph = { url = "git+https://gitlab.iscpif.fr/gargantext/haskell-igraph.git"; flake = false; };
    haskell-infomap = { url = "git+https://gitlab.iscpif.fr/gargantext/haskell-infomap.git"; flake = false; };

    # NP libs
    patches-class = { url = "git+https://gitlab.iscpif.fr/gargantext/patches-class.git"; flake = false; };
    patches-map = { url = "github:delanoe/patches-map"; flake = false; };

    # Accelerate Linear Algebra and specific instances
    accelerate = { url = "git+https://github.com/AccelerateHS/accelerate.git?submodules=1"; flake = false; };
    accelerate-arithmetic = { url = "github:alpmestan/accelerate-arithmetic"; flake = false; };
    # WARNING: using the fix-error branch
    accelerate-utility = { url = "git+https://gitlab.iscpif.fr/amestanogullari/accelerate-utility.git?ref=fix-error"; flake = false; };
    accelerate-llvm = { url = "github:AccelerateHS/accelerate-llvm"; flake = false; };
    # WARNING: using the alp/imap-static branch
    hmatrix = { url = "github:alpmestan/hmatrix/alp/imap-static"; flake = false; };
    # WARNING: using the alp/static branch
    sparse-linear = { url = "github:alpmestan/sparse-linear/alp/static"; flake = false; };
    wikiparsec = { url = "github:adinapoli/wikiparsec/adinapoli/support-ghc-947"; flake = false; };

    # WARNING: fork of https://github.com/boolexpr/boolexpr
    # WARNING: was merged upstream
    boolexpr = { url = "github:adinapoli/boolexpr/adinapoli/term-parsing"; flake = false; };

    # WARNING: fork of https://github.com/tomjaguarpaw/haskell-opaleye
    # Tracked in: https://gitlab.iscpif.fr/gargantext/haskell-gargantext/issues/317
    haskell-opaleye = { url = "github:garganscript/haskell-opaleye/tsquery-fixes"; flake = false; };

    # WARNING: using the more-exports branch
    servant-job = { url = "github:alpmestan/servant-job/more-exports"; flake = false; };

    servant-xml-conduit = { url = "git+https://gitlab.iscpif.fr/gargantext/servant-xml-conduit"; flake = false; };

    # WARNING: fork of https://github.com/facebook/duckling
    # using adinapoli/ghc947-compat branch
    # Tracked in: https://gitlab.iscpif.fr/gargantext/haskell-gargantext/issues/319
    duckling = { url = "github:adinapoli/duckling/adinapoli/ghc947-compat"; flake = false; };

    # WARNING: compat shim for duckling
    text16-compat = { url = "github:adinapoli/text16-compat"; flake = false; };

    # WARNING: fork of https://github.com/llvm-hs/llvm-hs
    # using adinapoli/llvm-12-ghc-947-compat branch
    # Tracked in: https://gitlab.iscpif.fr/gargantext/haskell-gargantext/issues/318
    llvm-hs = { url = "github:adinapoli/llvm-hs/adinapoli/llvm-12-ghc-947-compat"; flake = false; };

    eigen = { url = "github:chessai/eigen"; flake = false; };
  };

  # For printing the available outputs:
  #     $ nix -L flake show --allow-import-from-derivation
  # Note that it is very slow the first time,
  # because we chose to generate checks for each package of the closure
  # and that requires to generate a plan for each of them.
  outputs = { self, ... }@inputs:
    let
      # FIXME: darwin support is untested.
      supportedSystems = [
        "x86_64-linux"
        "x86_64-darwin"
        #"aarch64-linux"
        "aarch64-darwin"
      ];
    in
      inputs.flake-utils.lib.eachSystem supportedSystems (system:
        let
          pkgs = import inputs.nixpkgs {
            inherit system;
            config = inputs.haskellNix.config;
            overlays = [
              inputs.haskellNix.overlay
              (finalPkgs: previousPkgs: {

                graphviz = previousPkgs.graphviz.overrideAttrs (finalAttrs: previousAttrs: {
                  patches = (previousAttrs.patches or [ ]) ++ [
                    # Increase the YY_BUF_SIZE
                    # See https://gitlab.iscpif.fr/gargantext/haskell-gargantext/issues/290#note_9015
                    patches/graphviz/graphviz-yy-buf-size.diff
                  ];
                });

              })
            ];
          };

          # A library of very basic Nix expressions.
          lib = inputs.nixpkgs.lib;

          # A library of Nix expressions internal to haskell.nix.
          inherit (pkgs.haskell-nix) haskellLib;

          # Pinning the index-state of Hackage,
          # instead of using the latest known by haskell.nix,
          # removes haskell.nix from interfering too much into the reproducibility.
          # It also enables to materialize the plan-nix.
          index-state = haskellLib.parseIndexState (lib.readFile ./cabal.project);

          compiler-nix-name = "ghc947";

          # nix-tools/nix-tools/cabal2nix/Main.hs findCabalFiles
          # always prefers package.yaml over *.cabal,
          # but when the resulting cabal file is different
          # than a previously existing one,
          # the build fails with an error like this one:
          #     crawlerIsidore.cabal was modified manually, please use --force to overwrite.
          # Hence just remove this out-of-sync package.yaml.
          removePackageYaml = src: pkgs.symlinkJoin {
            name = "removePackageYaml-patched";
            paths = [ src ];
            postBuild = "rm $out/package.yaml";
          };

          # haskell.nix's main entry point
          project = pkgs.haskell-nix.cabalProject' {
            name = "gargantext";
            src = ./.;

            # By default plan evaluation is done on the build system.
            #evalSystem = "x86_64-linux";
            inherit compiler-nix-name;
            inherit index-state;

            # Materializing a project means recording
            # the nix files generated from the .cabal et al. files.
            # To update:
            #     $ nix run .#update-nix-cache-plan
            #materialized = nix/cache/plan;

            #compilerSelection = p: p.haskell.compiler;

            # Using inputMap for each source-repository-package of cabal.project
            # leverages `nix flake` to get their sha256 hashes
            # and check upstreams for updates.
            inputMap = {
              "https://github.com/AccelerateHS/accelerate-llvm.git" = inputs.accelerate-llvm;
              "https://github.com/AccelerateHS/accelerate.git" = inputs.accelerate;
              "https://github.com/MercuryTechnologies/ekg-json.git" = inputs.ekg-json;
              "https://github.com/adinapoli/boolexpr.git" = inputs.boolexpr;
              "https://github.com/adinapoli/duckling.git" = inputs.duckling;
              "https://github.com/adinapoli/llvm-hs.git" = inputs.llvm-hs;
              "https://github.com/adinapoli/text16-compat.git" = inputs.text16-compat;
              "https://github.com/adinapoli/wikiparsec.git" = inputs.wikiparsec;
              "https://github.com/alpmestan/accelerate-arithmetic.git" = inputs.accelerate-arithmetic;
              "https://github.com/alpmestan/hmatrix.git" = inputs.hmatrix;
              "https://github.com/alpmestan/servant-job.git" = inputs.servant-job;
              "https://github.com/alpmestan/sparse-linear.git" = inputs.sparse-linear;
              "https://github.com/chessai/eigen.git" = inputs.eigen;
              "https://github.com/delanoe/data-time-segment.git" = inputs.data-time-segment;
              "https://github.com/delanoe/patches-map" = inputs.patches-map;
              "https://github.com/garganscript/haskell-opaleye.git" = inputs.haskell-opaleye;
              "https://github.com/robstewart57/rdf4h.git" = inputs.rdf4h;
              "https://gitlab.iscpif.fr/amestanogullari/accelerate-utility.git" = inputs.accelerate-utility;
              "https://gitlab.iscpif.fr/gargantext/crawlers/arxiv-api.git" = inputs.crawlerArxiv;
              "https://gitlab.iscpif.fr/gargantext/crawlers/epo-proxy-api.git" = inputs.epo-api-client;
              "https://gitlab.iscpif.fr/gargantext/crawlers/hal.git" = inputs.crawlerHAL;
              "https://gitlab.iscpif.fr/gargantext/crawlers/isidore.git" = inputs.crawlerIsidore;
              "https://gitlab.iscpif.fr/gargantext/crawlers/istex.git" = inputs.crawlerISTEX;
              "https://gitlab.iscpif.fr/gargantext/crawlers/openalex.git" = inputs.openalex;
              "https://gitlab.iscpif.fr/gargantext/crawlers/pubmed.git" = inputs.crawlerPubMed;
              "https://gitlab.iscpif.fr/gargantext/gargantext-graph.git" = inputs.gargantext-graph;
              "https://gitlab.iscpif.fr/gargantext/haskell-gargantext-prelude" = inputs.gargantext-prelude;
              "https://gitlab.iscpif.fr/gargantext/haskell-igraph.git" = inputs.haskell-igraph;
              "https://gitlab.iscpif.fr/gargantext/haskell-infomap.git" = inputs.haskell-infomap;
              "https://gitlab.iscpif.fr/gargantext/hlcm.git" = inputs.hlcm;
              "https://gitlab.iscpif.fr/gargantext/iso639.git" = inputs.iso639;
              "https://gitlab.iscpif.fr/gargantext/patches-class.git" = inputs.patches-class;
              "https://gitlab.iscpif.fr/gargantext/servant-xml-conduit.git" = inputs.servant-xml-conduit;

              # Fixes needed for specific revisions
              # ===================================

              "https://gitlab.iscpif.fr/gargantext/crawlers/isidore.git/e9b594a4718acc06db037f6d3f429a90db76c267" = removePackageYaml inputs.crawlerIsidore;
              "https://gitlab.iscpif.fr/gargantext/haskell-gargantext-prelude/618f711a530df56caefbb1577c4bf3d5ff45e214" = removePackageYaml inputs.gargantext-prelude;
              "https://gitlab.iscpif.fr/gargantext/crawlers/istex.git/c0a08d62c40a169b7934ceb7cb12c39952160e7a" = removePackageYaml inputs.crawlerISTEX;
              "https://github.com/alpmestan/accelerate-arithmetic.git/a110807651036ca2228a76507ee35bbf7aedf87a" = pkgs.applyPatches {
                name = "accelerate-arithmetic-patched";
                src = inputs.accelerate-arithmetic;
                patches = [ patches/accelerate-arithmetic/0001-remove-test-using-removed-realBandedGramian.patch ];
              };
            };

            # Default project configuration.
            modules = [
              {
                packages.haskell-igraph.components.library = {
                  # The generated plan includes pkgs.igraph
                  # but pkgs.igraph.dev is also needed for igraph.h
                  libs = [ pkgs.igraph.dev ];
                  # Extra include needed because haskell-igraph's cbits
                  # use: #include <igraph.h>
                  # not: #include <igraph/igraph.h>
                  configureFlags = [ "--extra-include-dirs=${pkgs.igraph.dev}/include/igraph" ];
                };
              }
            ];

            # Variants to the default project configuration above.
            # They're accessed in the flake's outputs with their name prefixed.
            #     $ nix -L build .#static-x86_64-unknown-linux-musl:gargantext:exe:gargantext-phylo-profile
            # Or via project.projectCross:
            #     $ nix -L build .#project.projectVariants.static.projectCross.musl64.hsPkgs.postgresql-libpq.components.library
            flake.variants = {
              profiled = {
                modules = [
                  {
                    # Applies to all packages of the Haskell closure. For instance:
                    #     $ nix eval .#project.hsPkgs.containers.components.library.config.enableProfiling
                    #     false
                    #     $ nix eval .#project.projectVariants.profiled.hsPkgs.containers.components.library.config.enableProfiling
                    #     true
                    enableProfiling = true;
                    enableLibraryProfiling = true;
                  }
                ];
              };

              # For building static executables:
              #     $ nix -L build .#static-x86_64-unknown-linux-musl:gargantext:exe:gargantext-phylo-profile
              #
              # Note the use of the Musl cross compiling platform
              # to replace glibc which does not fully support static linking.
              #
              # For debugging static executables (Musl is not bug-to-bug compatible with glibc):
              #     $ nix -L develop .#static-x86_64-unknown-linux-musl:gargantext:exe:gargantext-phylo-profile
              #     [nix-shell] $ rm -rf source outputs
              #     [nix-shell] $ genericBuild # may fail at some point depending on the bug
              #     [nix-shell] $ cd source
              #     [nix-shell] $ eval "$configurePhase"
              #     [nix-shell] $ eval "$buildPhase"
              #     [nix-shell] $ (unset LD_LIBRARY_PATH; $EDITOR **/*.hs) # hack hack hack
              #     [nix-shell] $ eval "$buildPhase"
              #     [nix-shell] $ eval "$installPhase"
              #
              # See also https://github.com/nh2/static-haskell-nix # which:
              # > contains a large amount of per-package fixes for static builds
              # > for which we haven't found a way to integrate them cleanly into nixpkgs yet
              static = {
                modules = [
                  {
                    enableStatic = true;
                    #enableShared = false;

                    # Musl does not provide the thread-safe random_r()/initstate_r()
                    # so make randomVector fallback to a **non** thread-safe alternative, nrand48()
                    # See https://github.com/haskell-numerics/hmatrix/issues/279
                    #
                    # As a consequence, random numbers may not be random between threads
                    # as they now share a common state.
                    # See http://www.evanjones.ca/random-thread-safe.html
                    # > the best solution for truly portable applications
                    # > is to include your own random number generator implementation,
                    # > and not rely on the system's C library.
                    # See https://github.com/haskell-numerics/hmatrix/issues/307
                    packages.hmatrix.flags.no-random_r = true;

                    # Use pkg-config instead of pg_config to avoid:
                    #     $ nix -L build .#legacyPackages.x86_64-linux.project.projectVariants.static.projectCross.musl64.hsPkgs.postgresql-libpq.components.library
                    #     > Error: Setup: The program 'pg_config' is required but it could not be found
                    packages.postgresql-libpq.flags.use-pkg-config = true;
                  }
                ];
              };
            };

            flake.crossPlatforms = platforms:
              pkgs.lib.optionals pkgs.stdenv.hostPlatform.isx86_64 (
                # For building static executables.
                pkgs.lib.optionals pkgs.stdenv.hostPlatform.isLinux [
                  platforms.musl64
                ]
                # For cross compiling to Windows.
                # ++ [ platforms.mingwW64 ]
              );

            # Shell configuration shared by the default shell
            # and all shells from the flake.variants.
            shell = {
              # Force cabal-install (by setting CABAL_CONFIG=)
              # to use the packages selected by project.plan-nix.
              exactDeps = true;

              # Provision (in `ghc-pkg list`) the **dependencies** of the packages selected here,
              # which are not also selected here.
              # By default haskell.nix selects here all _local_ packages
              # (packages from both the packages and the source-repository-package stanzas)
              # which therefore excludes source-repository-packages from being provisioned.
              # Hence select only the _project_ packages (packages from the packages stanza).
              packages = ps: lib.attrValues (haskellLib.selectProjectPackages ps);

              # Add in this list any development tool needed
              # that is not expected to come from the developers' own system.
              nativeBuildInputs = [
                # Wraps cabal-install to remove source-repository-package stanzas,
                # otherwise it would download an use them,
                # likely because it does not know that those packages provisioned
                # in `ghc-pkg list` must be used in priority.
                (let cabal-install = project.tool "cabal" {
                  version = "latest";
                  #inherit index-state;
                  modules = [
                    {
                      # Preserve Bash completion for cabal.
                      packages.cabal-install.components.exes.cabal.postInstall = ''
                        mkdir -p $out/share/bash-completion
                        mv bash-completion $out/share/bash-completion/completions
                      '';
                    }
                  ];
                };
                in
                # symlinkJoin cabal-install to preserve share/bash-completion/
                pkgs.symlinkJoin {
                  name = "cabal-install";
                  paths = [ cabal-install ];
                  nativeBuildInputs = [ pkgs.makeWrapper ];
                  # Wrap cabal-install to make it use the local cabal.project.flake.
                  postBuild = ''
                    rm "$out"/bin/cabal
                    makeWrapper \
                      ${cabal-install}/bin/cabal \
                      "$out"/bin/cabal \
                      --add-flags --project-file=cabal.project.flake
                  '';
                })

                # Provide HLS because it is tighly coupled to the GHC version in use.
                # TODO: uncomment when in public binary caches.
                #(project.tool "haskell-language-server" { version = "latest"; })

                (project.tool "ghcid" { version = "latest"; })

                (project.tool "hlint" { version = "latest"; })
              ];

              # cabal.project.flake must be in the same directory than the original cabal.project
              # for the relative paths in the packages stanza to point to the correct subdirs.
              shellHook = ''
                ln -sf ${pkgs.runCommandLocal "cabal.project" {
                      nativeBuildInputs = with pkgs; [ gawk ];
                    }
                    # Strips source-repository-package stanzas from cabal.project.
                    ''
                      awk '/^[^ ]/{f=/^source-repository-package/} !f' >$out ${./cabal.project}
                    ''
                  } cabal.project.flake
              '';

              # If true, builds a Hoogle documentation index of all dependencies,
              # and provides a "hoogle" command to search the index.
              withHoogle = false;
            };
          };

          projectFlake = project.flake { };
        in {
          legacyPackages = pkgs // {
            # For exploring the project:
            #     $ nix --extra-experimental-features 'flakes repl-flake' repl .
            #     nix-repl> :lf .
            #     nix-repl> __attrNames legacyPackages.x86_64-linux.project
            #     [ "__overlay__" "appendModule" "appendOverlays" "args" "buildProject" "extend"
            #       "flake" "flake'" "getComponent" "getPackage" "ghcWithHoogle" "ghcWithPackages"
            #       "hsPkgs" "index-state-max" "makeConfigFiles" "options" "pkg-set" "pkgs"
            #       "plan-nix" "projectCoverageReport" "projectCross" "projectFunction"
            #       "projectModule" "projectVariants" "roots" "shell" "shellFor" "tool" "tools"
            #     ]
            project = project;
          };

          # For building a component of this project:
          #     $ nix -L build .#gargantext:exe:gargantext-phylo-profile
          packages = projectFlake.packages;

          # For entering the default development shell:
          #     $ nix -L develop
          #     $ cabal build --disable-optimization
          #
          # For entering the development shell variant "profiled":
          #     $ nix -L develop .#profiled
          #     $ cabal run gargantext-phylo-profile
          devShells = projectFlake.devShells;

          apps = projectFlake.apps // {
            # For updating nix/cache/plan
            #     $ nix run .#update-nix-cache-plan
            #     $ git commit -m "nix: update nix/cache/plan"
            update-nix-cache-plan = {
              type = "app";
              program = (pkgs.writeShellScript "update-nix-cache-plan" ''
                set -x
                ${project.plan-nix.passthru.generateMaterialized} nix/cache/plan
                git add --all nix/cache/plan
              '').outPath;
            };
            # Register the project's toolchain for compiler-nix-name
            # (but not the Haskell package dependencies of the project)
            # to prevent nix-collect-garbage from removing them from the Nix store.
            register-roots = {
              type = "app";
              program = (pkgs.writeShellScript "register-roots" ''
                nix-store --add-root nix/cache/roots --indirect --realise ${project.roots}
              '').outPath;
            };
          };

          # For running all checks (very slow):
          #     $ nix -L flake check
          # FIXME: actually run them all and disable or fix the broken ones.
          #
          # For running a specific check:
          #     $ nix -L build .#checks.x86_64-linux.duckling:test:duckling-test
          #     bat result/test-stdout
          # See names from:
          #     $ nix -L flake show --allow-import-from-derivation
          checks = projectFlake.checks // (
            # Ignore projectFlake.checks which only contain
            # this project test-suites (and flake.variants)
            # and generate instead a check for each test-suite
            # of each package in the closure of this project.
            let
              projectClosure = project.appendModule {
                # haskell.nix's applyComponents filters-out non-planned components.
                # Hence enable planned for all components of the project's Haskell closure.
                modules = [ { planned = true; } ];
              };
              testPackages = lib.mapAttrs (name: package:
                let
                  # Packages for which a plan cannot be computed
                  # even with jailbreaking .cabal's constraints.
                  brokenPackages = [
                    # fail (backjumping, conflict set: binary, hashable, text, unordered-containers)
                    "binary"
                    # fail (backjumping, conflict set: ghc, tasty-inspection-testing, time)
                    "random"
                    # fail (backjumping, conflict set: base, cassava, scientific)
                    "scientific"
                    # fail (backjumping, conflict set: base, ekg, servant-ekg)
                    "servant-ekg"
                    # fail (backjumping, conflict set: QuickCheck, aeson, criterion, criterion-measurement, splitmix)
                    "splitmix"
                    # fail (backjumping, conflict set: criterion, time, time:bench)
                    "time"
                    # fail (backjumping, conflict set: optparse-applicative, process, tasty, tasty-hunit, template-haskell, unix, optparse-applicative:process)
                    "unix"
                  ];
                  # Packages for which a plan can be computed
                  # by jailbreaking .cabal's constraints.
                  jailbreakPackages = [
                    "MissingH"
                    "Unique"
                    "cassava"
                    "cassava-conduit"
                    "filepath"
                    "full-text-search"
                    "fullstop"
                    "morpheus-graphql"
                    "morpheus-graphql-app"
                    "morpheus-graphql-client"
                    "morpheus-graphql-core"
                    "morpheus-graphql-server"
                    "mwc-random"
                    "system-filepath"
                    "vector"
                  ];
                  # The jailbreaking tool.
                  jailbreak-cabal = pkgs.haskell-nix.hackage-package {
                    name = "jailbreak-cabal";
                    version = "latest";
                    revision = "default";
                    compiler-nix-name = compiler-nix-name;
                    inherit index-state;
                  };
                in
                if package != null
                && (package.isHaskell or false)
                # Already provided by projectFlake.checks
                && !(package.isProject or false)
                # Some packages cannot be used:
                # error: The option `packages.BNFC.package.identifier.name' is used but not defined.
                && (package.components ? "library")
                # Some packages cannot be used:
                # error: The option `packages.GLFW-b.package.identifier.name' is used but not defined.
                && builtins.length (builtins.attrNames package.components.tests) != 0
                && !(lib.elem name brokenPackages)
                then
                  # This includes this project's packages
                  # and source-repository-package.
                  if projectClosure.hsPkgs.${name}.isLocal
                  then projectClosure.hsPkgs.${name}
                  else pkgs.haskell-nix.hackage-package {
                    name = package.identifier.name;
                    version = package.identifier.version;
                    revision = package.identifier.revision or "default";
                    compiler-nix-name = compiler-nix-name;
                    inherit index-state;
                    # Unfortunately I cannot find a shorter way
                    # to use jailbreak-cabal since haskell.nix
                    # only applies postPatch in builder/comp-builder.nix
                    # thus when building a component and not before building a plan.
                    # Hence copy and adapt modules/hackage-project.nix's src
                    src = lib.mkIf (lib.elem name jailbreakPackages) (
                      let
                        revision = package.identifier.revision or "default";
                        version = package.identifier.version;
                        tarball = pkgs.fetchurl {
                          url = "mirror://hackage/${name}-${version}.tar.gz";
                          inherit (pkgs.haskell-nix.hackage.${name}.${version}) sha256;
                        };
                        rev = pkgs.haskell-nix.hackage.${name}.${version}.revisions.${revision};
                        cabalFile = pkgs.fetchurl {
                          url = "https://hackage.haskell.org/package/${name}-${version}/revision/${toString rev.revNum}.cabal";
                          inherit (rev) sha256;
                        };
                        revSuffix = lib.optionalString (rev.revNum > 0) "-r${toString rev.revNum}";
                      in lib.mkOverride 1000 (pkgs.runCommand "${name}-${version}${revSuffix}-src" {
                        # Avoid readDir and readFile IFD functions looking for these project files in the hackage source
                        passthru.lookForCabalProject = false;
                      } (''
                        tmp=$(mktemp -d)
                        cd $tmp
                        tar xzf ${tarball}
                        mv "${name}-${version}" $out
                      '' + lib.optionalString (rev.revNum > 0) ''
                        cp ${cabalFile} $out/${name}.cabal
                      '' + ''
                        ${jailbreak-cabal.components.exes.jailbreak-cabal.exePath} $out/${name}.cabal
                      '') // {
                        # TODO remove once nix >=2.4 is widely adopted (will trigger rebuilds of everything).
                        # Disable filtering keeps pre ond post nix 2.4 behaviour the same.  This means that
                        # the same `alex`, `happy` and `hscolour` are used to build GHC.  It also means that
                        # that `tools` in the shell will be built the same.
                        filterPath = { path, ... }: path;
                      })
                    );
                  }
                else null
              ) projectClosure.hsPkgs;
            in
            haskellLib.mkFlakeChecks (haskellLib.collectChecks' testPackages)
          );

          #hydraJobs = projectFlake.hydraJobs;
        }
      );

  # Ask users to set Nix config entries in its local nix.conf.
  nixConfig = {
    # This sets the flake to use the IOG nix cache.
    # Nix should ask for permission before using it,
    # but remove it here if you do not want it to.
    extra-substituters = [
      "https://cache.iog.io"
      "https://static-haskell-nix.cachix.org"
    ];
    extra-trusted-public-keys = [
      "hydra.iohk.io:f/Ea+s+dFdN+3Y/G+FDgSq+a5NEWhJGzdjvKNGv0/EQ="
      "static-haskell-nix.cachix.org-1:Q17HawmAwaM1/BfIxaEDKAxwTOyRVhPG5Ji9K3+FvUU="
    ];

    # haskell.nix translates to Nix expressions:
    # - the build plan usually generated in `dist-newstyle/cache/plan.json` by `cabal configure`
    # - and the `.cabal`/`stack.yaml`/`package.yaml` files of projects.
    #
    # haskell.nix can either generate those Nix expressions on-demand
    # by calling its nix-tools' make-install-plan and cabal-to-nix,
    # hence importing them from a derivation (IFD).
    # Or import pre-generated files whenever project's materialized= attribute is set,
    # andthen no longer needs to allow IFD.
    #
    # Note that the projects for the checks of the closure
    # are never materialized, hence checking or showing the checks
    # always requires to allow IFD.
    allow-import-from-derivation = "true";
  };
}
